import sys

def add(*args):
    """批量添加参数（自动根据空格拆分）"""
    for arg in args:
        if isinstance(arg, str) and " " in arg:
            # 自动拆分空格分隔的参数（如 "--name 张三" → ["--name", "张三"]）
            sys.argv.extend(arg.split(" "))
        else:
            sys.argv.append(str(arg))

def set(target_arg: str, new_value):
    """
    设置 / 更新指定参数的值
    - 若参数存在且有值：覆盖旧值
    - 若参数存在但无值（flag）：插入新值
    - 若参数不存在：追加到末尾
    """
    argv = sys.argv
    new_value = str(new_value)

    i = 0
    while i < len(argv):
        if argv[i] == target_arg:
            # 参数存在
            if i + 1 < len(argv) and not argv[i + 1].startswith("-"):
                argv[i + 1] = new_value
            else:
                argv.insert(i + 1, new_value)
            return
        i += 1

    # 参数不存在，追加
    argv.extend([target_arg, new_value])


def set_batch(param_dict: dict[str, ...]) -> None:
    """批量设置参数"""
    for arg, value in param_dict.items():
        set(arg, value)


def get(key: str, default=None) -> str:
    """
    使用“裸参数名”获取参数值
    - key: 不带 - / --，如 "name"、"age"
    """
    argv = sys.argv

    long_opt = f"--{key}"
    short_opt = f"-{key}"

    for i, arg in enumerate(argv):
        if arg == long_opt or arg == short_opt:
            # 有 value
            if i + 1 < len(argv) and not argv[i + 1].startswith("-"):
                return argv[i + 1]
            # flag
            return True

    return default


def clear():
    """清空参数（自动保留脚本名）"""
    sys.argv = sys.argv[:1]

def reset(args_list):
    """完全替换参数（自动保留脚本名）"""
    sys.argv = [sys.argv[0]] + list(args_list)

def remove(target_arg: str):
    """
    删除指定参数及其对应的值（支持多次出现）
    """
    argv = sys.argv
    i = 0

    while i < len(argv):
        if argv[i] == target_arg:
            # 删除参数本身
            del argv[i]

            # 如果后一个元素存在，且它不是一个新的参数名，则认为是 value
            if i < len(argv) and not argv[i].startswith("-"):
                del argv[i]
            # 不递增 i，继续检查当前位置（防止连续参数）
        else:
            i += 1

if __name__ == '__main__':
    import argparse

    add("--name 张三", "-a 20")  # 自动拆分空格分隔的参数
    set("--name", "李四")
    set("-a", 67)

    # 解析参数（此时 argparse 会读取修改后的 sys.argv）
    parser = argparse.ArgumentParser()
    parser.add_argument("--name", type=str)
    parser.add_argument("-a", "--age", type=int)
    args = parser.parse_args()

    print(f"姓名：{args.name}，年龄：{args.age}")  # 姓名：张三，年龄：20


import argparse

def b_zip():
    parser = argparse.ArgumentParser()
    parser.add_argument("dir_path")
    args = parser.parse_args()

    from .Barchive import b_archive_zip
    b_archive_zip(args.dir_path)

def b_dirtree():
    parser = argparse.ArgumentParser()
    parser.add_argument("dir_path")
    args = parser.parse_args()

    from . import B_os
    B_os.dir_tree(args.dir_path)



import sys
import time
import threading
from typing import Iterable, Literal

from ..Butils import B_Color, B_Appearance, B_Background


class B_Tqdm:
    def __init__(
            self,
            range: int|Iterable = None,
            type: None|Literal['MB', 'KB'] = None,
            prefix: str = 'Processing',
            suffix: str = '',
            length: int = 20,
            fill: str = '█',
    ):
        """
        类似tqdm的进度条
        :param total: 总数
        :param prefix: 前缀
        :param suffix: 后缀
        :param length: 进度条长度(字符), 默认为20个字符长度
        :param fill: 填充字符
        """
        super().__init__()
        self.range = range
        if isinstance(range, (int, float, str)):
            self.range_len = int(range)
        elif isinstance(range, Iterable):
            self.range_len = len(range)
        else:
            self.range_len = float('inf')

        self.type = type
        self.prefix = prefix
        self.suffix = suffix
        self.length = length
        self.fill = fill
        self.start_time = 0
        self.current = 0

        self._lock = threading.Lock()

    def _format_time(self, seconds):
        """将秒数转换为mm:ss格式"""
        minutes = int(seconds // 60)
        seconds = int(seconds % 60)
        return f'{minutes:02}:{seconds:02}'

    def update(self, step=1, prefix=None, suffix=None, color:B_Color=B_Color.BLUE, appearance:B_Appearance=None, background:B_Background=None):
        with self._lock:
            pre_show = color.value
            if appearance is not None:
                pre_show += appearance.value
            if background is not None:
                pre_show += background.value

            if self.current == 0:
                self.start_time = time.time()
            if prefix is not None:
                self.prefix = prefix
            if suffix is not None:
                self.suffix = suffix

            # 更新进度
            self.current += step

            # 计算已用时间
            elapsed_time = time.time() - self.start_time
            used_time = self._format_time(elapsed_time)
            # 预估剩余时间
            if self.range_len != float('inf'):
                estimated_time = elapsed_time / self.current * (self.range_len - self.current) if self.current > 0 else 0
                rest_time = self._format_time(estimated_time)
            else:
                rest_time = None
            # 计算每秒处理的项数
            speed = self.current / elapsed_time if elapsed_time > 0 else 0

            # 更新进度条
            self._update(pre_show, self.current, self.range_len, used_time, rest_time, speed)

            # 进度条结束后换行
            if self.current >= self.range_len:
                sys.stdout.write('\n')
                sys.stdout.flush()

    def _update(self, pre_show, current, total, used_time, rest_time, speed):
        filled_length = int(self.length * current // total)
        bar = self.fill * filled_length + '-' * (self.length - filled_length) * len(self.fill)

        match self.type:
            case 'MB':
                style = 'MB'
                current = current / 1024**2
                total = total / 1024**2
                speed = speed / 1024**2
            case 'KB':
                style = 'KB'
                current = current / 1024
                total = total / 1024
                speed = speed / 1024
            case _:
                style = 'it'
                current = current
                total = total
                speed = speed
        if total != float('inf'):
            sys.stdout.write(f'\r{pre_show}{self.prefix} |{bar}|'
                             f' {current:.3g}/{total:.3g} {style} -> {used_time}<{rest_time} | {speed:.1f} {style}/s |'
                             f' {self.suffix}{B_Color.RESET.value}')
        else:
            sys.stdout.write(f'\r{pre_show}{self.prefix} | {current:.3g} {style} -> {used_time} | {speed:.1f} {style}/s |'
                             f' {self.suffix}{B_Color.RESET.value}')
        sys.stdout.flush()

    def __iter__(self):
        for item in self.range:
            yield item
            self.update()



from pathlib import Path
from datetime import datetime
import textwrap

#### 项目结构 ####
PROJECT_STRUCTURE = {
    # 模块文件
    "models": [
        "__init__.py",
    ],
    "train": [
        "__init__.py",
    ],
    "visualization": [
        "__init__.py",
    ],
    "utils": [
        "__init__.py",
    ],
    "configs": [
        "config.yaml",
    ],
    "scripts": [
        "run_pipeline_1.py",
        "run_pipeline_2.py",
        "run_pipeline_3.py",
    ],

    # 非模块文件
    "data": [],
    "logs": [],
    "outputs": [],
}

ROOT_FILES = [
    ".gitignore",
    "README.md",
    "requirements.txt",
]

#### 具体内容 ####
GITIGNORE_CONTENT = textwrap.dedent("""
    # Python
    __pycache__/
    *.pyc
    *.pyo
    *.pyd

    # virtual env
    .venv/
    venv/
    env/

    # logs
    logs/

    # outputs
    outputs/

    # dataset
    data/*

    # IDE
    .vscode/
    .idea/

    # mac
    .DS_Store
""")

INIT_CONTENT = textwrap.dedent("""


    __all__ = [

    ]
""")

CONFIG_YAML_CONTENT = textwrap.dedent("""
    # =========================
    # seed
    # =========================
    seed: 2026

    # =========================
    # Paths
    # =========================
    paths:
      output_dir: outputs
      log_dir: logs

    # =========================
    # Train
    # =========================
    train:
      batch_size: 64
      epochs: 50
      learning_rate: 0.001
      weight_decay: 0.0001
""")

PIPELINE_CONTENT = textwrap.dedent("""
    def main():
        pass

    if __name__ == "__main__":
        main()
""")


def _strip_once(s: str) -> str:
    if s.startswith("\n"):
        s = s[1:]
    if s.endswith("\n"):
        s = s[:-1]
    return s


def _make(file_path, content=None, overwrite=False):
    # 删去旧的, 创建新的
    if file_path.exists() and overwrite:
        print(f"Overwriting {file_path}...")
        file_path.unlink()
        if content:
            file_path.write_text(_strip_once(content) + "\n", encoding="utf-8")
        else:
            file_path.touch(exist_ok=True)
    # 保持不变
    elif file_path.exists():
        pass
    # 创建新的
    else:
        if content:
            file_path.write_text(_strip_once(content) + "\n", encoding="utf-8")
        else:
            file_path.touch(exist_ok=True)


def _makes(file: str, file_path: Path, overwrite=False):
    if file == "__init__.py":
        _make(file_path, content=INIT_CONTENT, overwrite=overwrite)

    elif file == ".gitignore":
        _make(file_path, content=GITIGNORE_CONTENT, overwrite=overwrite)

    elif file == "config.yaml":
        _make(file_path, content=CONFIG_YAML_CONTENT, overwrite=overwrite)

    elif file in ["run_pipeline_1.py", "run_pipeline_2.py", "run_pipeline_3.py"]:
        _make(file_path, content=PIPELINE_CONTENT, overwrite=overwrite)

    else:
        _make(file_path, overwrite=overwrite)


def b_create_project(project_dir: Path, overwrite=False):
    """
    创建项目目录结构
    """
    project_dir = Path(project_dir)

    # 创建根目录内文件夹
    for folder, files in PROJECT_STRUCTURE.items():
        folder_path = project_dir / folder
        folder_path.mkdir(parents=True, exist_ok=True)

        for file in files:
            file_path = (folder_path / file)
            _makes(file, file_path, overwrite=overwrite)

    # 创建根目录内文件
    for file in ROOT_FILES:
        file_path = project_dir / file
        _makes(file, file_path, overwrite=overwrite)

def _get_time():
    current_time = datetime.now()
    time_str = current_time.strftime("%Y_%m_%d-%H_%M_%S")
    return time_str
class _manager:
    def __init__(self, dir: Path):
        self.dir = Path(dir)

    def _get_count(self):
        # 统计self.run_dir_path下的文件夹数
        dirs = [1 for item in self.dir.iterdir() if item.is_dir()]
        count_1 = sum(dirs)

        # 统计文件夹的最大编号
        count_2 = 0
        for item in self.dir.iterdir():
            if item.is_dir():
                prefix = item.name.split("-", 1)[0]
                if prefix.isdigit():
                    count_2 = max(count_2, int(prefix))

        count = max(count_1, count_2)

        return count

    def create_dir(self, tag:str=None) -> tuple[Path, str]:
        '''
        :return: dir_path, dirname
        '''
        index = self._get_count() + 1
        time = _get_time()

        dir_path = self.dir / f'{index:03d}-{time}'
        if tag:
            dir_path = self.dir / f'{index:03d}-{time}-{tag}'
        dir_path.mkdir(parents=True, exist_ok=True)

        return dir_path, dir_path.name
class B_Project_Manager:
    def __init__(self, project_dir: Path):
        self.project_dir = Path(project_dir)

    def init_project(self, overwrite=False):
        b_create_project(self.project_dir, overwrite=overwrite)

    def get_logs(self):
        path = self.project_dir / "logs"
        assert path.exists(), "logs folder not exist"

        logs_manager = _manager(path)

        return path, logs_manager

    def get_outputs(self):
        path = self.project_dir / "outputs"
        assert path.exists(), "outputs folder not exist"

        outputs_manager = _manager(path)

        return path, outputs_manager

def main():
    # b_create_project("./")

    print(_get_time())

if __name__ == "__main__":
    main()


import time
class B_Timer:
    def __init__(self):
        '''
        >>> with B_Timer():
        >>>     ...
        '''
        self.start_time = 0
        self.end_time = 0

    def __enter__(self):
        self.start_time = time.time()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.end_time = time.time()
        print(f"[Timer] cost: {self.end_time - self.start_time:.4f}s")

if __name__ == '__main__':
    with B_Timer():
        time.sleep(2)
